/*
 * Logswan 2.1.12
 * Copyright (c) 2015-2021, Frederic Cambus
 * https://www.logswan.org
 *
 * Created:      2015-05-31
 * Last Updated: 2021-02-15
 *
 * Logswan is released under the BSD 2-Clause license.
 * See LICENSE file for details.
 *
 * SPDX-License-Identifier: BSD-2-Clause
 */

const char *methods_names[] = {
	"GET",
	"POST",
	"HEAD",
	"OPTIONS",
	"PUT",
	"DELETE",
	"TRACE",
	"CONNECT",
	"PATCH"
};

const char *protocols_names[] = {
	"HTTP/1.1",
	"HTTP/1.0",
	"HTTP/2.0",
	"HTTP/3"
};
